package ch.psi.daq.test.rest.controller;

import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

import org.junit.Test;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.result.MockMvcResultHandlers;
import org.springframework.test.web.servlet.result.MockMvcResultMatchers;

import ch.psi.daq.cassandra.reader.Ordering;
import ch.psi.daq.hazelcast.query.AggregationType;
import ch.psi.daq.rest.queries.PulseRangeQuery;
import ch.psi.daq.rest.queries.TimeRangeQuery;
import ch.psi.daq.test.rest.AbstractDaqRestTest;

import com.google.common.collect.Lists;
import com.google.common.collect.Sets;

/**
 * Tests the {@link DaqController} implementation. 
 */
public class DaqControllerTest extends AbstractDaqRestTest {

   private static final List<String> DEFAULT_PROPERTIES = Lists.newArrayList("channel", "pulseId");

   @Test
   public void testPulseRangeQuery() throws Exception {
      PulseRangeQuery request = new PulseRangeQuery(
            Ordering.DESC,
            Lists.newArrayList(), // DummyQueryProcessor simply returns a fixed list
            Sets.newLinkedHashSet(DEFAULT_PROPERTIES),
            false,
            AggregationType.index,
            null,
            100l,
            101l
            );
      
      String content = mapper.writeValueAsString(request);

      this.mockMvc
         .perform(MockMvcRequestBuilders.post("/pulserange")
               .contentType(MediaType.APPLICATION_JSON)
               .content(content))
         
               .andDo(MockMvcResultHandlers.print())
               .andExpect(MockMvcResultMatchers.status().isOk())
               .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
               .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
               .andExpect(MockMvcResultMatchers.jsonPath("$[0].pulseId").value(100))
               .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel").value("testChannel1"));
   }
   
   @Test
   public void testTimeRangeQuery() throws Exception {
      
      long startTime = new Date().getTime();
      long endTime = startTime + TimeUnit.SECONDS.toMillis(1);
      TimeRangeQuery request = new TimeRangeQuery(
            Ordering.ASC,
            Lists.newArrayList(), // DummyQueryProcessor simply returns a fixed list
            Sets.newLinkedHashSet(DEFAULT_PROPERTIES),
            false,
            AggregationType.index,
            null, // aggregations
            startTime, // startMillis
            0,
            null,
            endTime,
            0,
            null);
      
      String content = mapper.writeValueAsString(request);
      System.out.println(content);
      
      this.mockMvc
      .perform(MockMvcRequestBuilders.post("/timerange")
            .contentType(MediaType.APPLICATION_JSON)
            .content(content))
            
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel").value("testChannel1"));
   }
}
